﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Augmented_Reality {

  public partial class FormMain : EZ_Builder.UCForms.FormPluginMaster {

    EZ_Builder.UCForms.FormCameraDevice _cameraControl;
    byte                                _originalGridTransparency;

    public FormMain() {

      InitializeComponent();

      detach();
    }

    private void FormMain_FormClosing(object sender, FormClosingEventArgs e) {

      detach();
    }

    public override void SendCommand(string windowCommand, params string[] values) {

      if (windowCommand.Equals("attach", StringComparison.InvariantCultureIgnoreCase))
        attach();
      else if (windowCommand.Equals("detach", StringComparison.InvariantCultureIgnoreCase))
        detach();
      else
        base.SendCommand(windowCommand, values);
    }

    public override object[] GetSupportedControlCommands() {

      List<object> cmds = new List<object>();

      cmds.Add("Attach");
      cmds.Add("Detach");

      return cmds.ToArray();
    }

    private void btnAttach_Click(object sender, EventArgs e) {

      if (_cameraControl == null)
        attach();
      else
        detach();
    }

    void detach() {

      if (_cameraControl != null) {

        EZ_Builder.Invokers.SetAppendText(tbLog, true, "Detaching from {0}", _cameraControl.Text);

        _cameraControl.Camera.OnNewFrame -= Camera_OnNewFrame;

        _cameraControl.Camera.GridTransparency = _originalGridTransparency;
        
        _cameraControl.Camera.DisableDetectedRectangleDisplay = true;

        _cameraControl = null;
      }

      EZ_Builder.Invokers.SetText(btnAttach, "Attach");
    }

    void attach() {

      detach();

      Control [] cameras = EZ_Builder.EZBManager.FormMain.GetControlByType(typeof(EZ_Builder.UCForms.FormCameraDevice));

      if (cameras.Length == 0) {

        EZ_Builder.Invokers.SetAppendText(tbLog, true, "There are no camera controls in this project.");

        return;
      }

      _cameraControl = (EZ_Builder.UCForms.FormCameraDevice)cameras[0];

      _cameraControl.Camera.OnNewFrame += Camera_OnNewFrame;

      _originalGridTransparency = _cameraControl.Camera.GridTransparency;
      
      _cameraControl.Camera.GridTransparency = 0;

      _cameraControl.Camera.DisableDetectedRectangleDisplay = true;

      EZ_Builder.Invokers.SetAppendText(tbLog, true, "Attached to: {0}", _cameraControl.Text);

      EZ_Builder.Invokers.SetText(btnAttach, "Detach");
    }

    void Camera_OnNewFrame() {

      if (_cameraControl == null)
        return;

      using (Graphics g = Graphics.FromImage(_cameraControl.Camera.GetOutputBitmap.ToManagedImage(false))) {

        if (EZ_Builder.Scripting.VariableManager.GetVariable("$CameraIsTracking") == "0") {

          g.DrawString("No object detected", SystemFonts.CaptionFont, Brushes.Red, 0, 0);

          return;
        }

        int objectX = Convert.ToInt32(EZ_Builder.Scripting.VariableManager.GetVariable("$CameraObjectX"));
        int objectY = Convert.ToInt32(EZ_Builder.Scripting.VariableManager.GetVariable("$CameraObjectY"));
        int objectWidth = Convert.ToInt32(EZ_Builder.Scripting.VariableManager.GetVariable("$CameraObjectWidth"));
        int objectHeight = Convert.ToInt32(EZ_Builder.Scripting.VariableManager.GetVariable("$CameraObjectHeight"));

        g.DrawImage(pictureBox1.Image, objectX, objectY, objectWidth, objectHeight);
      }
    }

    private void btnLoadImage_Click(object sender, EventArgs e) {

      using (OpenFileDialog ofd = new OpenFileDialog()) {

        if (ofd.ShowDialog() != System.Windows.Forms.DialogResult.OK)
          return;

        pictureBox1.Load(ofd.FileName);
      }
    }
  }
}
